/******************************************************************************/
/* CodeSign, by LoRd_MuldeR <MuldeR2@GMX.de>                                  */
/* This work has been released under the CC0 1.0 Universal license!           */
/******************************************************************************/

#include "common.h"
#include <sys/time.h>
#include <io.h>
#include <fcntl.h>

#define WIN32_LEAN_AND_MEAN 1
#include <Windows.h>

#include <openssl/crypto.h>

void print_logo(const char *const app_name)
{
    SetErrorMode(SEM_FAILCRITICALERRORS | SEM_NOGPFAULTERRORBOX);
    _setmode(_fileno(stdin), _O_BINARY);
    fprintf(stderr, "CodeSign - %s [" __DATE__ "], by LoRd_MuldeR <mulder2@gmx.de>\n", app_name);
    fprintf(stderr, "using %.15s\n\n", OpenSSL_version(OPENSSL_VERSION));
    fflush(stderr);
}

void print_license(void)
{
    fputs("This work has been released under the CC0 1.0 Universal license!\n", stderr);
    fputs("Please see http://www.muldersoft.com for additional information.\n\n", stderr);
}

char *convert_wchar_to_UTF8(const wchar_t *str_w)
{
    char *str_utf8 = NULL;
    if (str_w)
    {
        int bytes = WideCharToMultiByte(CP_UTF8, 0, str_w, -1, NULL, 0, NULL, NULL);
        if (bytes > 0)
        {
            str_utf8 = malloc(bytes);
            if(str_utf8)
            {
                if (WideCharToMultiByte(CP_UTF8, 0, str_w, -1, str_utf8, bytes, NULL, NULL) == 0) 
                {
                    free(str_utf8);
                    return NULL;
                }
            }
        }
    }
    return str_utf8;
}

char *read_line_from_file(FILE *const file, const int trim)
{
    char buffer[4096], *line;
    size_t len;
    while(!(feof(file) || ferror(file)))
    {
        if((line = fgets(buffer, sizeof(buffer), file)))
        {
            if (trim)
            {
                while (*line && isspace(*line))
                {
                    ++line;
                }
            }
            len = strlen(line);
            while (len > 0)
            {
                if ((line[len - 1U] != '\r') && (line[len - 1U] != '\n'))
                {
                    break;
                }
                line[--len] = '\0';
            }
            if (trim)
            {
                while (len > 0)
                {
                    if (!isspace(line[len - 1U]))
                    {
                        break;
                    }
                    line[--len] = '\0';
                }
            }
            if (len > 0)
            {
                return strdup(line);
            }
        }
    }
    return NULL;
}

UI64_T get_current_time(void)
{
    struct timeval t;
    if(gettimeofday(&t, NULL) == 0)
    {
        return (((UI64_T)t.tv_sec) * 1000000ULL) + ((UI64_T)t.tv_usec);
    }
    return 0ULL;
}

void store_uint64(unsigned char *const buffer, const UI64_T value)
{
    buffer[0U] = (BYTE)(value >> 56);
    buffer[1U] = (BYTE)(value >> 48);
    buffer[2U] = (BYTE)(value >> 40);
    buffer[3U] = (BYTE)(value >> 32);
    buffer[4U] = (BYTE)(value >> 24);
    buffer[5U] = (BYTE)(value >> 16);
    buffer[6U] = (BYTE)(value >>  8);
    buffer[7U] = (BYTE)(value >>  0);
}

UI64_T load_uint64(const unsigned char *const buffer)
{
    UI64_T value = 0U;
    value |= ((UI64_T)buffer[0U]) << 56;
    value |= ((UI64_T)buffer[1U]) << 48;
    value |= ((UI64_T)buffer[2U]) << 40;
    value |= ((UI64_T)buffer[3U]) << 32;
    value |= ((UI64_T)buffer[4U]) << 24;
    value |= ((UI64_T)buffer[5U]) << 16;
    value |= ((UI64_T)buffer[6U]) <<  8;
    value |= ((UI64_T)buffer[7U]) <<  0;
    return value;
}

const unsigned char* load_resource_data(const wchar_t *const name, unsigned int *const length)
{
    HRSRC resource = FindResourceW(NULL, name, RT_RCDATA);
    if (resource)
    {
        *length = SizeofResource(NULL, resource);
        if (*length > 0U)
        {
            HGLOBAL data = LoadResource(NULL, resource);
            if (data)
            {
                return (const unsigned char*) LockResource(data);
            }
        }
    }
    *length = 0U;
    return NULL;
}

int force_flush(FILE *const file)
{
    const int result = fflush(file);
    FlushFileBuffers((HANDLE)_get_osfhandle(_fileno(file)));
    return (result == 0);
}

void zero_memory(void *const ptr, const size_t size)
{
    SecureZeroMemory(ptr, size);
}
